<?php

/**
 * E-goi.com
 *
 * @title      E-Goi Multi-channel Marketing
 * @package    E-Goi
 * @copyright  Copyright (c) 2012-2025 E-Goi - https://www.e-goi.com
 */

namespace Egoi\Marketing\Controller\Adminhtml\Account;

use Egoi\Marketing\Helper\Data;
use Magento\Backend\App\Action;

/**
 * Class Transactional
 *
 * @package Egoi\Marketing\Controller\Adminhtml\Account
 */
class Transactional extends \Magento\Backend\App\Action
{

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $_scopeConfig;

    /**
     * @var Data
     */
    protected $egoiHelper;

    /**
     * Transactional constructor.
     *
     * @param  Data  $helper
     * @param  \Magento\Framework\App\Config\ScopeConfigInterface  $scopeConfig
     * @param  Action\Context  $context
     */
    public function __construct(
        \Egoi\Marketing\Helper\Data $helper,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        Action\Context $context
    ) {

        parent::__construct($context);

        $this->egoiHelper = $helper;
        $this->_scopeConfig = $scopeConfig;
    }

    public function execute()
    {

        try {
            $salesSender = $this->_scopeConfig->getValue('trans_email/ident_general/name');
            $salesEmail = $this->_scopeConfig->getValue('trans_email/ident_general/email');

            $message = new \Laminas\Mail\Message;
            $message->setBody(
                'If you are receiving this message,'.
                ' everything seems to be ok with your SMTP configuration'
            );

            $message->setFrom($salesEmail, $salesSender)
                ->addTo($salesEmail, $salesSender)
                ->setSubject('E-Goi / Magento - Test');

            $transport = new \Laminas\Mail\Transport\Smtp();
            $options = new \Laminas\Mail\Transport\SmtpOptions($this->egoiHelper->getSMTPOptions());

            if ($domain = $this->_scopeConfig->getValue('egoi/transactional/domain')) {
                $message->getHeaders()->addHeaderLine('X-Domain', $domain);
            }

            $transport->setOptions($options);
            $transport->send($message);
        } catch (\Exception $e) {
            $this->egoiHelper->getLogger()->critical($e->getMessage());
            $this->messageManager->addErrorMessage('Error Testing your Settings: '.$e->getMessage());
        }

        return $this->resultRedirectFactory->create()
            ->setUrl($this->_redirect->getRefererUrl());
    }
}
