<?php

/**
 * E-goi.com
 *
 * @title      E-Goi Multi-channel Marketing
 * @package    E-Goi
 * @copyright  Copyright (c) 2012-2025 E-Goi - https://www.e-goi.com
 */

namespace Egoi\Marketing\Controller\Adminhtml\Autoresponders;

use Magento\Backend\App\Action\Context;
use Magento\Framework\App\Action\HttpPostActionInterface as HttpPostActionInterface;
use Magento\Framework\App\Request\DataPersistorInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Filter\FilterInput;
use Magento\Framework\Registry;
use Magento\Framework\Stdlib\DateTime\Filter\Date;
use Magento\Framework\Stdlib\DateTime\TimezoneInterface;

/**
 * Class Save
 *
 * @package Egoi\Marketing\Controller\Adminhtml\Autoresponders
 */
class Save extends \Egoi\Marketing\Controller\Adminhtml\Autoresponders
{

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $_scopeConfig;

    /**
     * @var Date
     */
    protected $_dateFilter;

    /**
     * @var TimezoneInterface
     */
    private $localeDate;

    /**
     * @var
     */
    protected $_coreDate;

    /**
     * @var \Magento\Catalog\Model\ProductFactory
     */
    protected $_productFactory;

    /**
     * @param  Action\Context  $context
     * @param  \Magento\Framework\View\Result\PageFactory  $resultPageFactory
     * @param  Registry  $registry
     * @param  \Magento\Catalog\Model\ProductFactory  $productFactory
     * @param  Date  $dateFilter
     * @param  \Magento\Framework\Stdlib\DateTime\DateTime  $dateTime
     * @param  \Egoi\Marketing\Model\AutorespondersFactory  $autorespondersFactory
     * @param  \Magento\Framework\App\Config\ScopeConfigInterface  $scopeConfigInterface
     * @param  \Magento\Backend\Model\View\Result\ForwardFactory  $resultForwardFactory
     * @param  \Magento\Framework\View\Result\LayoutFactory  $resultLayoutFactory
     */
    public function __construct(
        TimezoneInterface $localeDate,
        Context $context,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory,
        Registry $registry,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        Date $dateFilter,
        \Magento\Framework\Stdlib\DateTime\DateTime $dateTime,
        \Egoi\Marketing\Model\AutorespondersFactory $autorespondersFactory,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfigInterface,
        \Magento\Backend\Model\View\Result\ForwardFactory $resultForwardFactory,
        \Magento\Framework\View\Result\LayoutFactory $resultLayoutFactory
    ) {

        parent::__construct(
            $context,
            $resultPageFactory,
            $registry,
            $autorespondersFactory,
            $resultForwardFactory,
            $resultLayoutFactory
        );

        $this->localeDate = $localeDate;
        $this->_coreDate = $dateTime;
        $this->_dateFilter = $dateFilter;
        $this->_scopeConfig = $scopeConfigInterface;
        $this->_productFactory = $productFactory;
    }

    /**
     * Save action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function execute()
    {

        parent::execute();
        /** @var \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
        $resultRedirect = $this->resultRedirectFactory->create();
        // check if data sent
        $data = $this->getRequest()->getPostValue();
        if ($data) {
            $id = $this->getRequest()->getParam('id');

            $model = $this->_coreRegistry->registry('egoi_autoresponder');

            if ( ! $model->getId() && $id) {
                $this->messageManager->addErrorMessage(__('This Autoresponder no longer exists.'));

                return $resultRedirect->setPath('*/*/');
            }

            if ( ! $this->getRequest()->getParam('from_date')) {
                $data['from_date'] = $this->localeDate->formatDate();
            }
            $filterValues = ['from_date' => $this->_dateFilter];
            if ($this->getRequest()->getParam('to_date')) {
                $filterValues['to_date'] = $this->_dateFilter;
            }
            $inputFilter = new FilterInput(
                $filterValues,
                [],
                $data
            );
            $data = $inputFilter->getUnescaped();

            if ( ! isset($data['store_ids'])) {
                $data['store_ids'] = [0];
            }
            if (array_search(0, $data['store_ids']) !== false) {
                $data['store_ids'] = [];
            }
            $data['store_ids'] = implode(',', $data['store_ids']);

            if ( ! isset($data['payment_method'])) {
                $data['payment_method'] = [];
            }
            $data['payment_method'] = implode(',', $data['payment_method']);

            if ( ! isset($data['shipping_method'])) {
                $data['shipping_method'] = [];
            }
            $data['shipping_method'] = implode(',', $data['shipping_method']);

            if (array_search(10000, $data['customer_groups']) !== false) {
                $data['customer_groups'] = [];
            }
            $data['customer_groups'] = implode(',', $data['customer_groups']);

            $model->setData('controller', true);

            // try to save it
            try {
                $model->addData($data);
                $model->save();

                $this->messageManager->addSuccessMessage(__('You saved the Autoresponder.'));
                $this->_getSession()->setFormData(false);

                if ($this->getRequest()->getParam('back')) {
                    return $resultRedirect->setPath(
                        '*/*/edit',
                        [
                            'id'     => $model->getId(),
                            'tab_id' => $this->getRequest()->getParam('active_tab'),
                        ]
                    );
                }

                return $resultRedirect->setPath('*/*/');
            } catch (\Magento\Framework\Exception\LocalizedException $e) {
                $this->messageManager->addErrorMessage($e->getMessage());
            } catch (\RuntimeException $e) {
                $this->messageManager->addErrorMessage($e->getMessage());
            } catch (\Exception $e) {
                $this->messageManager->addExceptionMessage(
                    $e,
                    __('Something went wrong while saving the Autoresponder.')
                );
            }

            $this->_getSession()->setFormData($data);

            return $resultRedirect->setPath(
                '*/*/edit',
                [
                    'id'     => $model->getId(),
                    'tab_id' => $this->getRequest()->getParam('active_tab'),
                ]
            );
        }

        return $resultRedirect->setPath('*/*/');
    }

}
