<?php

namespace Egoi\Marketing\Controller\Callback;

use Egoi\Marketing\Model\Egoi;

/**
 * Class Index
 *
 * @package Egoi\Marketing\Controller\Callback
 */
class Index extends \Magento\Framework\App\Action\Action
{

    /**
     * @var \Egoi\Marketing\Model\SubscriberFactory
     */
    protected $subscriberFactory;

    /**
     * @var Egoi
     */
    protected $egoi;

    /**
     * @param  \Egoi\Marketing\Model\SubscriberFactory  $subscriberFactory
     * @param  \Magento\Framework\App\Action\Context  $context
     * @param  Egoi  $egoi
     */
    public function __construct(
        \Egoi\Marketing\Model\SubscriberFactory $subscriberFactory,
        \Magento\Framework\App\Action\Context $context,
        Egoi $egoi
    ) {

        $this->egoi = $egoi;
        $this->subscriberFactory = $subscriberFactory;
        parent::__construct($context);

        $this->processData();
    }

    public function processData()
    {

        $json = file_get_contents('php://input');
        $data = json_decode($json, true);
        if (empty($data)) {
            die(':(');
        }

        if ($this->egoi->getHelper()->getScopeConfig()->isSetFlag('egoi/test/log')) {
            $this->egoi->getLogger()->debug($json);
        }

        try {
            if (is_array($data['events'])) {
                foreach ($data['events'] as $event) {
                    if (in_array($event['action'],
                        [
                            'subscription', 'edit_subscription', 'unsubscribe', 'resubscription',
                        ]
                    )
                    ) {
                        $remoteSubscriber = $this->egoi->getSubscriber($event['list_id'], $event['contact']['base'][0]);

                        if ( ! empty($remoteSubscriber['base']['email'])) {
                            $email = $remoteSubscriber['base']['email'];
                            $uid = $remoteSubscriber['base']['contact_id'];
                            $subscriber = $this->subscriberFactory->create();

                            if ($remoteSubscriber['base']['status'] == Egoi::EGOI_STATUS_SUBSCRIBED
                                && $remoteSubscriber['base']['email_status'] == Egoi::EGOI_STATUS_SUBSCRIBED
                            ) {
                                $subscriber->subscribe($email, $uid);
                            } elseif ($remoteSubscriber['base']['status'] != Egoi::EGOI_STATUS_SUBSCRIBED
                                || $remoteSubscriber['base']['email_status'] != Egoi::EGOI_STATUS_SUBSCRIBED
                            ) {
                                $subscriber->unsubscribe($email, $uid);
                            }
                        }
                    }
                }
            }
        } catch (\Exception $e) {
            $this->egoi->getLogger()->warning($e->getMessage());
            die($e->getMessage());
        }

        die('ok');
    }

    public function execute()
    {
    }

}
