<?php

/**
 * E-goi.com
 *
 * @title      E-Goi Multi-channel Marketing
 * @package    E-Goi
 * @copyright  Copyright (c) 2012-2025 E-Goi - https://www.e-goi.com
 */

namespace Egoi\Marketing\Helper;

/**
 *
 */
class Qero extends Data
{

    const OPERATION_BUY = 'BUY';

    const OPERATION_RETURN = 'RETURN';

    const OPERATION_CAPTIVE = 'CAPTIVE';

    /**
     *
     */
    const QERO_ENDPOINT = 'Https://qero-cdp.e-goi.com/public-api/';

    /**
     * @param  \Magento\Catalog\Model\Product  $product
     *
     * @return false|void
     */
    public function qeroProductView(\Magento\Catalog\Model\Product $product)
    {

        if ( ! $this->isEnabled()) {
            return false;
        }

        if ($product->getData('qero_sync') >= $this->dateTime->gmtDate('Y-m-d 00:00:00')) {
            return false;
        }

        $productExists = $this->makeQeroRequest('product/?identifier='.$product->getSku());

        if ($productExists) {
            $this->makeQeroRequest(
                'product/'.$productExists[0]['product_id'],
                $this->getProductDataArray($product),
                'PUT'
            );
        } else {
            $this->makeQeroRequest('product', $this->getProductDataArray($product), 'POST');
        }

        $product->setData('qero_sync', $this->dateTime->gmtDate('Y-m-d'))->save();
    }

    /**
     * @param  \Magento\Catalog\Model\Product  $product
     *
     * @return array
     */
    public function getProductDataArray($product)
    {

        $data = [];
        $data['name'] = $product->getName();
        $data['identifier'] = $product->getSku();
        if ($product->getData('ean')) {
            $data['ean'] = $product->getData('ean');
        }
        if ($product->getAttributeText('manufacturer')) {
            $data['manufacturer'] = $product->getAttributeText('manufacturer');
        }
        $data['stock'] = $product->getQty();
        $data['price'] = round($product->getPrice() * 100);
        $data['description'] = $product->getShortDescription();
        $data['img_url'] = $this->imageHelper->init($product, 'product_page_main_image')
            ->setImageFile($product->getImage())
            ->getUrl();

        return $data;
    }

    /**
     * @param  \Magento\Sales\Model\Order  $order
     * @param  string  $type
     *
     * @return false|mixed
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\PaymentException
     */
    public function queroOrderRegister(\Magento\Sales\Model\Order $order, string $type)
    {

        if ( ! $this->isEnabled()) {
            return false;
        }

        $result = false;

        if ($type == self::OPERATION_CAPTIVE) {
            $phone = $this->getPhoneNumber();
            $order->setData('qero_telephone', $phone);
            $qeroPoints = (int) $this->checkoutSession->getQuote()->getData('qero_points');
            $order->setData('qero_points', $qeroPoints);
        } else {
            $phone = $order->getData('qero_telephone');
            $qeroPoints = $order->getData('qero_points');
        }

        if ( ! $this->canUsePoints()) {
            $qeroPoints = 0;
        }

        $client = $this->makeQeroRequest('client/?secondary_id='.$phone);
        if ( ! $client) {
            $data = [];
            $data['secondary_id'] = $phone;
            $data['email'] = $order->getCustomerEmail();
            $data['name'] = $order->getCustomerName();
            $client = $this->makeQeroRequest('client', $data, 'POST');
        } else {
            $client = $client[0];
        }

        if (empty($client['status']) || $client['status'] != 'Ativo') {
            $this->getLogger()->debug('Client request: '.$phone.' | '.print_r($client, true));

            return false;
        }

        if ($type == self::OPERATION_CAPTIVE) {
            $captive = $this->makeQeroRequest(
                'balance/captivate/'.$client['terciary_id'],
                ['credit' => $qeroPoints],
                'POST'
            );

            if (empty($captive['hash'])) {
                throw new \Magento\Framework\Exception\LocalizedException(__('Error Processing Loyalty Points'));
            } else {
                $order->setData('qero_hash_captive', $captive['hash']);
            }
        }

        if ($type == self::OPERATION_BUY || $type == self::OPERATION_RETURN) {
            $hash = $this->makeQeroRequest('client/'.$client['terciary_id'].'/balance');

            $data = [];
            $data['secondary_id'] = $order->getIncrementId().' - '.$type;
            $data['type'] = $type;
            $data['terciary_id'] = $client['terciary_id'];
            $data['external_date'] = date('Y-m-d H:i:s');
            $data['amount_gross'] = (int) $order->getBaseGrandTotal() * 100 + $qeroPoints;
            $data['amount_net'] = ((int) $order->getBaseGrandTotal() - $order->getBaseTaxAmount()) * 100 + $qeroPoints;
            $data['identity_name'] = $this->scopeConfig->getValue('egoi/qero/identity');
            $data['hash'] = $hash['hash'];
            $data['credit_out'] = $qeroPoints;
        }

        if ($type == self::OPERATION_RETURN) {
            $data['external_return_id'] = $order->getIncrementId().' - BUY';
            $this->makeQeroRequest(
                'balance/captivate/'.$client['terciary_id'].'/cancel',
                [
                    'hash' => $order->getData('qero_hash_captive'),
                ],
                'POST'
            );
        }

        if ($type == self::OPERATION_BUY || $type == self::OPERATION_RETURN) {
            /** @var \Magento\Sales\Api\Data\OrderItemInterface $item */
            foreach ($order->getAllVisibleItems() as $item) {
                $product = $this->productFactory->create()->load($item->getProductId());
                $cats = $product->getCategoryCollection();
                $resultArray = [];
                foreach ($cats as $cat) {
                    if ( ! empty($cat->getName())) {
                        $resultArray[] = str_replace(';', '', $cat->getName());
                    }
                }
                $resultArray[] = str_replace(';', '', $item->getSku());

                $data['products'][] = [
                    'description' => $product->getSortDescription(),
                    'id'          => implode(';', $resultArray),
                    'separator'   => count($resultArray) > 0 ? ';' : '',
                    'amount_unit' => [
                        'gross' => round($item->getBasePriceInclTax() * 100, 2),
                        'net'   => round(($item->getBasePrice() - $item->getBaseDiscountAmount()) * 100, 2),
                    ],
                    'quantity'    => $item->getQtyOrdered(),
                    'IVA'         => (int) $item->getTaxPercent(),
                ];
            }

            $result = $this->makeQeroRequest('client/'.$client['terciary_id'].'/movement', $data, 'POST');

            if ( ! empty($result['terciary_id']) && $type == self::OPERATION_BUY && ! empty($hash['hash'])) {
                $order->setData('qero_processed', 1);
                $order->setData('qero_hash', $hash['hash']);
            } elseif (empty($result['terciary_id']) && $type == self::OPERATION_BUY && $qeroPoints > 0) {
                throw new \Magento\Framework\Exception\PaymentException(
                    __('Error Processing Qero Points [127]'.print_r($result, true))
                );
            }
        }

        $this->customerSession->setData('qero_applied_points', 0);
        $this->customerSession->setData('qero_points', 0);

        return $result;
    }

    /**
     * @return bool
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function isEnabled()
    {

        return $this->scopeConfig->isSetFlag(
            'egoi/qero/enable',
            'store',
            $this->storeManagerInterface->getStore()->getId()
        );
    }

    /**
     * @param  \Magento\Sales\Model\Order  $order
     *
     * @return false|mixed
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     * @throws \Magento\Framework\Exception\PaymentException
     */
    public function queroOrderCancel(\Magento\Sales\Model\Order $order)
    {

        if ( ! $this->canUsePoints()) {
            return false;
        }

        $this->customerSession->setData('qero_applied_points', 0);
        $this->customerSession->setData('qero_points', 0);
        $phone = $this->getPhoneNumber($order->getData('qero_telephone'));
        $qeroPoints = $order->getData('qero_points');

        if ( ! $qeroPoints) {
            return false;
        }

        return $this->queroOrderRegister($order, \Egoi\Marketing\Helper\Qero::OPERATION_RETURN);
    }

    /**
     * @param $phone
     *
     * @return false|string
     */
    public function getPhoneNumber($phone = null)
    {

        $countryId = $this->getScopeConfig()
            ->getValue('general/store_information/country_id');
        if ( ! $phone) {
            $phone = $this->getCustomerModel()->getData('customer_telephone');
        }
        if ( ! $phone) {
            return false;
        }

        $prefix = \Egoi\Marketing\Model\Subscriber::getPrefixForCountry($countryId);

        if (stripos($phone, '-') !== false) {
            $number = preg_replace('/[^0-9-]/', '', $phone);
        } else {
            $number = preg_replace('/\D/', '', $phone);
            $number = ltrim($number, $prefix);
            $number = $prefix.'-'.$number;
        }

        return ltrim($number, 0);
    }

    /**
     * @return bool
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function canUsePoints()
    {

        return $this->isEnabled()
            && $this->scopeConfig->isSetFlag(
                'egoi/qero/points',
                'store',
                $this->storeManagerInterface->getStore()->getId()
            );
    }

    /**
     * @return false|float|int
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getMaxPointsAllowed()
    {

        if ( ! $this->getQeroPoints()
            ||
            ! $this->getQeroBalance()
        ) {
            return false;
        }

        $total = $this->getCheckoutSession()->getQuote()->getGrandTotal() * 100;
        $points = $this->getQeroPoints();
        $newValue = ceil($points * 100 / $total);

        $fPoints = $points * $this->getQeroBalance()['max_points_use_percentage'] / 100;
        $fQuote = $total * $this->getQeroBalance()['max_points_use_percentage'] / 100;

        return min($fPoints, $fQuote);
    }

    /**
     * @return mixed
     */
    public function getQueryAppliedPoints()
    {

        return $this->customerSession->getData('qero_applied_points');
    }

    /**
     * @return string
     */
    public function showEmptyPhoneMessage()
    {

        $emptyPhone = $this->scopeConfig->isSetFlag('egoi/qero/empty_phone');

        if ( ! $emptyPhone) {
            return '';
        }

        if ( ! $this->getCustomerId()) {
            return $this->scopeConfig->getValue('egoi/qero/empty_phone_message_guest');
        }

        $phone = $this->getCustomerModel()->getData('customer_telephone');

        if ( ! $phone) {
            return $this->scopeConfig->getValue('egoi/qero/empty_phone_message_customer');
        }

        return '';
    }

    /**
     * @return false|int|mixed
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getQeroPoints()
    {

        if ( ! $this->isEnabled()) {
            return 0;
        }

        if ( ! $this->customerSession->getData('qero_points')
            ||
            ! $this->getQeroBalance()
        ) {
            $phone = $this->getPhoneNumber();
            if ( ! $phone) {
                return false;
            }

            $client = $this->makeQeroRequest('client/?secondary_id='.$phone);

            if ( ! empty($client[0]['points'])) {
                $hash = $this->makeQeroRequest('client/'.$client[0]['terciary_id'].'/balance');

                $this->customerSession->setData('qero_points', $client[0]['points']);
                $this->customerSession->setData('qero_balance', $hash);
            }
        }

        return $this->customerSession->getData('qero_points');
    }

    /**
     * @return mixed
     */
    public function getQeroBalance()
    {

        return $this->customerSession->getData('qero_balance');
    }

    /**
     * @param $endpoint
     * @param  array  $data
     * @param  string  $method
     *
     * @return false|mixed
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function makeQeroRequest($endpoint, array $data = [], string $method = 'GET')
    {

        if ($this->scopeConfig->isSetFlag('egoi/test/log')) {
            $this->getLogger()->info("----------------   START   --------------------");
            $this->getLogger()->info("----------------   METHOD  --------------------");
            $this->getLogger()->info($method);
            $this->getLogger()->info("----------------  ENDPOINT --------------------");
            $this->getLogger()->info($endpoint);
        }

        $curl = curl_init();

        $apiKey = $this->scopeConfig->getValue(
            'egoi/qero/api',
            'store',
            $this->storeManagerInterface->getStore()->getId()
        );

        $suffix = self::QERO_ENDPOINT;
        if ($this->scopeConfig->getValue('egoi/qero/endpoint')) {
            $suffix = rtrim($this->scopeConfig->getValue('egoi/qero/endpoint'), '/').'/';
        }

        $dataEndpoint = [
            CURLOPT_URL            => $suffix.$endpoint,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING       => "",
            CURLOPT_MAXREDIRS      => 5,
            CURLOPT_TIMEOUT        => 5,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST  => strtoupper($method),
            CURLOPT_HTTPHEADER     => [
                "APIKEY: ".$apiKey,
                "Content-Type: application/json",
            ],
        ];

        curl_setopt($curl, CURLOPT_VERBOSE, true);

        $streamVerboseHandle = fopen('php://temp', 'w+');
        curl_setopt($curl, CURLOPT_STDERR, $streamVerboseHandle);

        if ($data) {
            $dataEndpoint[CURLOPT_POSTFIELDS] = json_encode($data);
        }

        curl_setopt_array($curl, $dataEndpoint);

        $response = json_decode(curl_exec($curl), true);
        curl_close($curl);

        if ($response === false && $this->scopeConfig->isSetFlag('egoi/test/log')) {
            $this->getLogger()->info("----------------  ERROR RESP -------------------");
            $this->getLogger()->info(
                "cUrl error ".curl_errno($curl).'| '.htmlspecialchars(curl_error($curl))
            );
        }

        if ($this->scopeConfig->isSetFlag('egoi/test/log')) {
            $this->getLogger()->info("----------------  DATA SENT --------------------");
            $this->getLogger()->info(print_r($data, true));
            $this->getLogger()->info("----------------   RESPONSE  -------------------");
            $this->getLogger()->info(print_r($response, true));
            $this->getLogger()->info("----------------     END     -------------------");
        }

        return $response;
    }

}
