<?php

namespace Egoi\Marketing\Plugin;

/**
 * Class Subscriber
 *
 * @package Egoi\Marketing\Plugin
 */
class Subscriber
{

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var \Egoi\Marketing\Model\SubscriberFactory
     */
    protected $subscriberFactory;

    /**
     * @var \Egoi\Marketing\Helper\Data
     */
    protected $helper;

    /**
     * @param  \Egoi\Marketing\Helper\Data  $helper
     * @param  \Egoi\Marketing\Model\SubscriberFactory  $subscriberFactory
     * @param  \Magento\Framework\App\Config\ScopeConfigInterface  $scopeConfig
     */
    public function __construct(
        \Egoi\Marketing\Helper\Data $helper,
        \Egoi\Marketing\Model\SubscriberFactory $subscriberFactory,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    ) {

        $this->helper = $helper;
        $this->scopeConfig = $scopeConfig;
        $this->subscriberFactory = $subscriberFactory;
    }

    /**
     * @param  \Magento\Newsletter\Model\Subscriber  $subject
     * @param                                      $result
     *
     * @return bool
     */
    public function afterIsSubscribed(\Magento\Newsletter\Model\Subscriber $subject, $result)
    {

        if ($subject->getInCallBack() || $this->helper->getRegistry()->registry('egoi_in_callback')) {
            return $result;
        }

        return $this->subscriberFactory->create()
            ->loadByEmail($subject->getEmail())
            ->isSubscribed();
    }

    /**
     * @param  \Magento\Newsletter\Model\Subscriber  $subject
     * @param                                      $result
     *
     * @return mixed
     */
    public function afterSubscribe(\Magento\Newsletter\Model\Subscriber $subject, $result)
    {

        if ($subject->getInCallBack() || $this->helper->getRegistry()->registry('egoi_in_callback')) {
            return $result;
        }
        try {
            $this->subscriberFactory->create()
                ->setData('from_plugin', true)
                ->subscribe($subject->getEmail(), false, $subject->getStoreId());
        } catch (\Exception $e) {
            $this->helper->getLogger()->critical($e->getMessage());
        }

        return $result;
    }

    /**
     * @param  \Magento\Newsletter\Model\Subscriber  $subject
     * @param                                      $result
     *
     * @return mixed
     */
    public function afterSubscribeCustomerById(\Magento\Newsletter\Model\Subscriber $subject, $result)
    {

        if ($subject->getInCallBack() || $this->helper->getRegistry()->registry('egoi_in_callback')) {
            return $result;
        }
        try {
            $this->subscriberFactory->create()
                ->setData('from_plugin', true)
                ->subscribe($subject->getEmail(), false, $subject->getStoreId());
        } catch (\Exception $e) {
            $this->helper->getLogger()->critical($e->getMessage());
        }

        return $result;
    }

    /**
     * @param  \Magento\Newsletter\Model\Subscriber  $subject
     * @param                                      $result
     *
     * @return mixed
     */
    public function afterUnsubscribe(\Magento\Newsletter\Model\Subscriber $subject, $result)
    {

        if ($subject->getInCallBack() || $this->helper->getRegistry()->registry('egoi_in_callback')) {
            return $result;
        }

        try {
            $this->subscriberFactory->create()
                ->setData('from_plugin', true)
                ->unsubscribe($subject->getEmail());
        } catch (\Exception $e) {
            $this->helper->getLogger()->critical($e->getMessage());
        }

        return $result;
    }

    /**
     * @param  \Magento\Newsletter\Model\Subscriber  $subject
     * @param                                      $result
     *
     * @return mixed
     */
    public function afterUnsubscribeCustomerById(\Magento\Newsletter\Model\Subscriber $subject, $result)
    {

        if ($subject->getInCallBack() || $this->helper->getRegistry()->registry('egoi_in_callback')) {
            return $result;
        }

        try {
            $this->subscriberFactory->create()
                ->setData('from_plugin', true)
                ->unsubscribe($subject->getEmail());
        } catch (\Exception $e) {
            $this->helper->getLogger()->critical($e->getMessage());
        }

        return $result;
    }
}
