<?php

namespace Egoi\Marketing\Plugin;

use \Magento\Newsletter\Model\SubscriptionManager;
use \Magento\Newsletter\Model\Subscriber;

/**
 *
 */
class SubscriberManager
{

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var \Egoi\Marketing\Model\SubscriberFactory
     */
    protected $subscriberFactory;

    private \Egoi\Marketing\Helper\Data $egoiHelper;

    /**
     * @param  \Egoi\Marketing\Helper\Data  $helper
     * @param  \Egoi\Marketing\Model\SubscriberFactory  $subscriberFactory
     * @param  \Magento\Framework\App\Config\ScopeConfigInterface  $scopeConfig
     */
    public function __construct(
        \Egoi\Marketing\Helper\Data $helper,
        \Egoi\Marketing\Model\SubscriberFactory $subscriberFactory,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    ) {

        $this->scopeConfig = $scopeConfig;
        $this->egoiHelper = $helper;
        $this->subscriberFactory = $subscriberFactory;
    }

    /**
     * @param  SubscriptionManager  $subject
     * @param  Subscriber  $result
     * @param  string  $email
     * @param  int  $storeId
     *
     * @return Subscriber
     */
    public function afterSubscribe(
        SubscriptionManager $subject,
        Subscriber $result,
        string $email,
        int $storeId
    ): Subscriber {

        if ($this->egoiHelper->getRegistry()->registry('egoi_in_callback')) {
            return $result;
        }

        try {
            $this->subscriberFactory->create()
                ->setData('from_plugin', true)
                ->subscribe($email, false, $storeId);
        } catch (\Exception $e) {
        }

        return $result;
    }

    /**
     * @param  SubscriptionManager  $subject
     * @param  Subscriber  $result
     * @param  string  $email
     * @param  int  $storeId
     * @param  string  $confirmCode
     *
     * @return Subscriber
     */
    public function afterUnsubscribe(
        SubscriptionManager $subject,
        Subscriber $result,
        string $email,
        int $storeId,
        string $confirmCode
    ): Subscriber {

        if ($this->egoiHelper->getRegistry()->registry('egoi_in_callback')) {
            return $result;
        }

        try {
            $this->subscriberFactory->create()
                ->setData('from_plugin', true)
                ->unsubscribe($email, false, $storeId);
        } catch (\Exception $e) {
        }

        return $result;
    }

    /**
     * @param  SubscriptionManager  $subject
     * @param  Subscriber  $result
     * @param  int  $customerId
     * @param  int  $storeId
     *
     * @return Subscriber
     */
    public function afterSubscribeCustomer(
        SubscriptionManager $subject,
        Subscriber $result,
        int $customerId,
        int $storeId
    ): Subscriber {

        if ($this->egoiHelper->getRegistry()->registry('egoi_in_callback')) {
            return $result;
        }
        try {
            $this->subscriberFactory->create()->setData('from_plugin', true)
                ->subscribe($result->getEmail(), false, $storeId);
        } catch (\Exception $e) {
        }

        return $result;
    }

    /**
     * @param  SubscriptionManager  $subject
     * @param  Subscriber  $result
     * @param  int  $customerId
     * @param  int  $storeId
     *
     * @return Subscriber
     */
    public function afterUnsubscribeCustomer(
        SubscriptionManager $subject,
        Subscriber $result,
        int $customerId,
        int $storeId
    ): Subscriber {

        if ($this->egoiHelper->getRegistry()->registry('egoi_in_callback')) {
            return $result;
        }

        try {
            $this->subscriberFactory->create()->setData('from_plugin', true)
                ->unsubscribe($result->getEmail(), false, $storeId);
        } catch (\Exception $e) {
        }

        return $result;
    }
}
