<?php

/**
 * E-goi.com
 *
 * @title      E-Goi Multi-channel Marketing
 * @package    E-Goi
 * @copyright  Copyright (c) 2012-2025 E-Goi - https://www.e-goi.com
 */

namespace Egoi\Marketing\Plugin;

use Magento\Framework\Mail\TransportInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;

/**
 * Class TransportPlugin
 *
 * @package Egoi\Marketing\PLugin
 */
class TransportPlugin
{

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var \Egoi\Marketing\Helper\Data
     */
    protected $egoiHelper;

    /**
     * TransportPlugin constructor.
     *
     * @param  \Egoi\Marketing\Helper\Data  $helper
     * @param  ScopeConfigInterface  $scopeConfigInterface
     */
    public function __construct(
        \Egoi\Marketing\Helper\Data $helper,
        ScopeConfigInterface $scopeConfigInterface
    ) {

        $this->scopeConfig = $scopeConfigInterface;
        $this->egoiHelper = $helper;
    }

    /**
     * @param  TransportInterface  $subject
     * @param  \Closure  $proceed
     *
     * @return mixed|void
     */
    public function aroundSendMessage(
        TransportInterface $subject,
        \Closure $proceed
    ) {

        if ( ! $this->scopeConfig->isSetFlag('egoi/transactional/enable')) {
            return $proceed();
        } else {
            try {
                /** @var \Magento\Framework\Mail\Message $message */
                $message = $subject->getMessage();
                $message = \Laminas\Mail\Message::fromString($message->getRawMessage())->setEncoding('utf-8');

                $transport = new \Laminas\Mail\Transport\Smtp();
                $options = new \Laminas\Mail\Transport\SmtpOptions($this->egoiHelper->getSMTPOptions());

                if ($domain = $this->scopeConfig->getValue('egoi/transactional/domain')) {
                    $message->getHeaders()->addHeaderLine('X-Domain', $domain);
                }

                $transport->setOptions($options);
                $transport->send($message);
            } catch (\Exception $e) {
                $this->egoiHelper->getLogger()->critical($e->getMessage());
            }
        }
    }

}
