<?php

namespace Egoi\Marketing\Setup;

use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\UpgradeDataInterface;
use Magento\Framework\Setup\UpgradeSchemaInterface;
use Magento\Framework\Setup\SchemaSetupInterface;
use Magento\Framework\DB\Ddl\Table;

/**
 * Class UpgradeData
 *
 * @package Egoi\Marketing\Setup
 */
class UpgradeSchema implements UpgradeSchemaInterface
{

    /**
     * {@inheritdoc}
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     */
    public function upgrade(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {

        $connection = $setup->getConnection();

        if (version_compare($context->getVersion(), '3.6.0', '<')) {
            $tableName = $setup->getTable('sales_order');
            if ($connection->isTableExists($tableName)) {
                if ( ! $connection->tableColumnExists($tableName, 'egoi_synced')) {
                    $connection->addColumn(
                        $tableName,
                        'egoi_synced',
                        [
                            'type'     => Table::TYPE_SMALLINT,
                            'nullable' => false,
                            'default'  => 0,
                            'comment'  => 'Needs Sync',
                        ]
                    );
                }
            }

            $tableName = $setup->getTable('egoi_subscribers');
            if ($connection->isTableExists($tableName)) {
                if ( ! $connection->tableColumnExists($tableName, 'needs_sync')) {
                    $connection->addColumn(
                        $tableName,
                        'needs_sync',
                        [
                            'type'     => Table::TYPE_BOOLEAN,
                            'nullable' => false,
                            'default'  => 0,
                            'comment'  => 'Needs Sync',
                        ]
                    );
                }

                if ( ! $connection->tableColumnExists($tableName, 'last_sync')) {
                    $connection->addColumn(
                        $tableName,
                        'last_sync',
                        [
                            'type'     => Table::TYPE_DATETIME,
                            'nullable' => true,
                            'comment'  => 'Last Sync Time',
                        ]
                    );
                }
            }
        }

        if (version_compare($context->getVersion(), '3.5.0', '<')) {
            try {
                $setup->run(
                    "ALTER TABLE `{$setup->getTable('egoi_autoresponders')}` ADD COLUMN `payment_method` varchar(255) DEFAULT NULL"
                );
                $setup->run(
                    "ALTER TABLE `{$setup->getTable('egoi_autoresponders')}` ADD COLUMN `shipping_method` varchar(255) DEFAULT NULL"
                );
            } catch (\Exception $e) {
            }
        }

        if (version_compare($context->getVersion(), '3.5.1', '<')) {
            try {
                $setup->run(
                    "ALTER TABLE `{$setup->getTable('egoi_autoresponders_events')}` ADD COLUMN `extra` varchar(255) DEFAULT NULL"
                );
                $setup->run(
                    "ALTER TABLE `{$setup->getTable('egoi_autoresponders_events')}` CHANGE COLUMN `message` `message` varchar(255) DEFAULT NULL"
                );
                $setup->run(
                    "ALTER TABLE `{$setup->getTable('egoi_autoresponders_events')}` ADD INDEX `extra` (`extra`) comment '';"
                );
            } catch (\Exception $e) {
            }
        }

        if (version_compare($context->getVersion(), '3.6.0', '<')) {
            try {
                $setup->run(
                    " 
                    CREATE TABLE `{$setup->getTable('egoi_conversions')}` (
                      `conversion_id` int(10) unsigned NOT NULL AUTO_INCREMENT,
                      `utm_term` varchar(255) DEFAULT NULL,
                      `utm_campaign` varchar(255) DEFAULT NULL,
                      `eg_sub` varchar(255) DEFAULT NULL,
                      `eg_cam` varchar(255) DEFAULT NULL,
                      `eg_list` varchar(255) DEFAULT NULL,
                      `order_id` varchar(50) DEFAULT NULL,
                      `order_amount` decimal(12,4) DEFAULT NULL,
                      `email` varchar(255) DEFAULT NULL,
                      `created_at` datetime DEFAULT NULL,
                      PRIMARY KEY (`conversion_id`)
                    ) ENGINE=InnoDB DEFAULT CHARSET=utf8 
                    "
                );
            } catch (\Exception $e) {
            }
        }
    }
}